#ifndef BUS_TYPES_H
#define BUS_TYPES_H

#include "basic_protocol.h"
#include "request.h"
#include "bounded_queue.h"
#include "bounded_set.h"
#include <tlm.h>

typedef airline::request* TRANS_TYPE;
typedef basic_protocol::action_t PAYLD_TYPE;


// The tokens that will be traveling on the chanels 
typedef basic_protocol::basic_request< TRANS_TYPE, PAYLD_TYPE > token_t;

// Non-blocking get interface
typedef tlm::tlm_nonblocking_get_if < token_t* > nb_get_if_t;

// Non-blocking put interface
typedef tlm::tlm_nonblocking_put_if < token_t* > nb_put_if_t;


// Incoming ports
typedef sc_port < nb_get_if_t > nb_in_port_t;


// Outgoing ports
typedef sc_port< nb_put_if_t > nb_out_port_t; 


// Parametrized tlm tag for communicating over the TLM channels
typedef tlm::tlm_tag< token_t* > tlm_tag_t;


/**
 * Comparison function for objects of type request. Used to define
 * the set container for our bounded queue.
 */
struct comp_id {
  bool operator()(const TRANS_TYPE o1, const TRANS_TYPE o2) {
    return o1->get_request_id() < o2->get_request_id();
  }
};



/**
 * Comparison function for objects of type request. We need it so
 * that we can use a priority queue for the pending requests.
 */
struct comp_priority {
  bool operator()(const TRANS_TYPE o1, const TRANS_TYPE o2) {
    return o1->get_priority() <= o2->get_priority();
  }
};


typedef user::bounded_queue<TRANS_TYPE, comp_priority> bpq_t; // Bounded Priority Queue
typedef user::bounded_set<TRANS_TYPE, comp_id> bis_t; // Bounded ID Set

#endif
