/**
 * Defines the functionality of the input modules for the system
 **/

#ifndef IO_MODULE_H
#define IO_MODULE_H

#include <systemc.h>
#include <tlm.h>
#include "bus_types.h"
#include "basic_module.h"
#include "basic_protocol.h"
#include "tlm.h"
#include "test_protocol.h"
#include "bounded_queue.h"

#ifdef PROPERTY_2
#include "property2.h"
class property2;
#endif

using namespace basic_protocol;
using user::basic_module;
using tlm::tlm_tag;

namespace airline{

  class io_module : 
    public sc_module,
    public basic_module,
    public virtual test_if<token_t, status_t>
  {
    
  public: 
    // The constructor
    io_module (sc_module_name module_name, unsigned int q_size );

    SC_HAS_PROCESS(io_module);

    // Ouput to the master module
    nb_out_port_t out_port_master; 
    
    // Input port from the master
    nb_in_port_t in_port_master; 
    
    // The IO sends tokens to the test generator through this port
    test_initiator_port< token_t, status_t > test_out_port;
    
    // Incoming requests from the test generator come through this port
    sc_export< test_if< token_t, status_t > > test_in_port;

    // Implementation of the function from test_if, needed for the
    // export above
    status_t receive_transaction(const token_t*);

    // Receive tokens from the master module to present to the user for selection
    void get_requests_for_selection();

    // Overwrite the virtual callback
    virtual void before_end_of_elaboration();

    // Module clock
    sc_in_clk clk;




  protected:
    // Main thread
    void do_it();

    // The queues
    bpq_t incoming_requests;
    
    // Size of the queues
    unsigned int queue_size;
    
    // There is something in the incoming queue
    sc_event inc_queue_nonempty;

    // SC_THREAD that sends new requests to the master
    void send_new_requests();

#ifdef PROPERTY_2
    // Verification
    vect_mon_t* prop2;
#endif

  }; //class

}; //namespace

#endif
