#include "main.h"


int sc_main(int argc, char** argv) {
  
  unsigned int io_q_size = 5;
  unsigned int master_q_size = 5;
  unsigned int planner_q_size = 5;

  // Command line parameters
  unsigned int sim_time = 0;
  bool enable_verification = 0;
  unsigned int num_monitors = 0;
  

  if (argc < 2) {
    printf("Use %s sim_time verif numb_monitors, where \n", argv[0]);
    printf("\t sim_time is in miliseconds\n");
    printf("\t ver is in {0,1} and disables/enables verification\n");
    printf("\t num_monitors is the number of monitors\n");
    exit(1);
  }

  else {
    printf("Arguments: \n");
    for (int i = 0; i < argc; i++) {
      printf("%d -> %s\n", i, argv[i]);
    }
    
    sim_time = atoi(argv[1]);

    if (argc > 2) {
      enable_verification = atoi(argv[2]);
    }

    if (argc > 3) {
      num_monitors = atoi(argv[3]);
    }

    printf("Simulation time = \t %d ms\n", sim_time);
    printf("Verifications is \t %s\n", (enable_verification? "ON" : "OFF"));
    printf("Number of monitors is \t %d\n", num_monitors);
  }
  
  
  // The modules
  master_module master("master_module", master_q_size);
  flight_planner planner("flight_planner", planner_q_size);
  io_module io1("io1", io_q_size);
  user::test_generator gen("test_generator");

  // Declare the communication channels
  basic_channel<TRANS_TYPE,PAYLD_TYPE> c_master_planner("master_planner", 4);
  basic_channel<TRANS_TYPE,PAYLD_TYPE> c_planner_master("planner_master", 4);
  basic_channel<TRANS_TYPE,PAYLD_TYPE> c_io_master("io_master", 4);
  basic_channel<TRANS_TYPE,PAYLD_TYPE> c_master_io("master_io", 4);

  // The clocks
  sc_clock fast_clock("fast_clock", 10, SC_NS, 0.5, 1, SC_NS);
  sc_clock slow_clock("slow_clock", 20, SC_NS, 0.5, 1, SC_NS);

  // Connect the exports
  gen.io1_out_port(io1.test_in_port);
  io1.test_out_port(gen.io1_in_port);


  //Connect the modules
  io1.out_port_master(c_io_master);
  master.in_port_io1(c_io_master);

  io1.in_port_master(c_master_io);
  master.out_port_io1(c_master_io);

  master.out_port_planner(c_master_planner);
  planner.in_from_master(c_master_planner);

  master.in_port_planner(c_planner_master);
  planner.out_to_master(c_planner_master);
  
  
  // Connect the clocks
  gen.clk(fast_clock);
  io1.clk(fast_clock);
  master.clk(slow_clock);
  planner.clk(slow_clock);


#ifdef OBSERVER_CODE
  // The observer instantiates the monitors
  if (enable_verification) {
    printf("Instantiating monitors\n");
    new my_observer(&planner, num_monitors);
  }
#endif

  printf("Starting simulation\n");
  sc_start(sim_time, SC_US);
  printf("Finished simulation\n");
  
  struct timeval tim; 
  struct rusage ru;        
  getrusage(RUSAGE_SELF, &ru);        
  tim=ru.ru_utime;        
  double t=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;        
  tim=ru.ru_stime;        
  t+=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;

  fprintf(stderr, "getrusage time: %.4f seconds\n", t);
  fprintf(stderr, "Duration: %d ms, monitor: %s\n", sim_time, (enable_verification? "yes":"no"));
  if (enable_verification) {
    fprintf(stderr, "number of monitors: %d\n", num_monitors);
  }

  return 0;
}
