/**
 * Defines the datatypes of the tokens that will be travelling through
 * the communication channels. Inspired by basic_protocol.h from the
 * examples that accompany the TLM source code.
 *
 * Initial creation date: September 2007
 */

#ifndef BASIC_PROTOCOL_H
#define BASIC_PROTOCOL_H

#include <systemc.h>
#include "request.h"

/**
 * Basic protocol definitions.
 */
namespace basic_protocol {

  // Action requests from the initiator modules
  enum action_t {
    NEW,
    PLAN,
    PLANDONE,
    UPDATED,
    CONNECT,
    SELECT,
    PAY,
    CONFIRM,
    CANCEL
  };

  // Confirmations to the initiator modules
  enum status_t {
    ERROR,
    SUCCESS
  };

  // Packaged request from initiator to target
  template< typename REQUEST, typename PAYLD > 
  class basic_request {

  public: 
    
    REQUEST request; // The request that we are refering to
    PAYLD   payload; // Additional data that the target may need

    /**
     * A constructor with explicit parameters.
     */
    basic_request(REQUEST t, PAYLD p): request(t), payload(p) {
    }

    /**
     * A constructor with default parameters. The user will have to
     * update those later.
     */     
    basic_request(): request(new airline::request()), payload (CANCEL) {
    }


    /**
     * A constructor only with the request id. 
     */
    basic_request(REQUEST t): request(t), payload(NULL) {
    }
   
    
    /**
     * Standard getter for the request id.
     */
    REQUEST get_request() const { return request; }
    
    
    /**
     * Standard getter for the payload.
     */
    PAYLD get_payload() const { return payload; }

    /**
     * Pretty print
     */
    static const char* to_string(PAYLD p) {
      switch ( p ) {
      case NEW : return "NEW";
      case PLAN : return "PLAN";
      case PLANDONE: return "PLANDONE";
      case UPDATED : return "UPDATED";
      case CONNECT : return "CONNECT";
      case CANCEL : return "CANCEL";
      default : return "OTHER";
      }
    }

  }; // basic request

} // end namespace

/***
template< typename REQUEST, typename PAYLD >
ostream &operator<<( ostream& fsout,
		     const basic_protocol::basic_request<REQUEST,PAYLD> &req) {
  switch (req.get_action()) {
  case basic_protocol::NEW : 
    fsout << "NEW " << req.get_request_id();
    break;

  case basic_protocol::PLAN : 
    fsout << "PLAN " << req.get_request_id();
    break;

  case basic_protocol::CONNECT : 
    fsout << "CONNECT " << req.get_request_id();
    break;

  case basic_protocol::SELECT : 
    fsout << "SELECT " << req.get_request_id();
    break;

  case basic_protocol::PAY : 
    fsout << "PAY " << req.get_request_id();
    break;

  case basic_protocol::CONFIRM : 
    fsout << "CONFIRM " << req.get_request_id();
    break;

  case basic_protocol::CANCEL : 
    fsout << "CANCEL " << req.get_request_id();
    break;

  default: 
    fsout << "ERROR: Unknown action: " << req.get_action() 
	  << " for transaction " << req.get_request_id();
    break;
  } //switch

  return fsout;
} // operator

**/




#endif
