#ifndef BOUNDED_SET_H
#define BOUNDED_SET_H

#include <set>
#include <algorithm>
#include <iostream>

namespace user {

  template < typename OBJ_TYPE, typename comp>
  class bounded_set: 
    public std::set<OBJ_TYPE, comp> {
    
  public:
    typedef std::set<OBJ_TYPE, comp> set_t;
    typedef std::pair<typename set_t::iterator, bool> pairset_t;
    
    // Constructor
    bounded_set(unsigned int size = 0):
      set_t(),
      s_size(size)
    {}
    
    // Guarded insert
    pairset_t insert(OBJ_TYPE o) {
      if (set_t::size() == s_size) {
	fprintf(stderr, "ERROR: Bounded set is full when attempting to add an element\n");
	exit(2);
      }
      
      return set_t::insert(o);
    }
    
    // Standard setter
    void set_size(unsigned int new_size) {
      s_size = new_size;
    }
    

    // Pretty print
    void print_me() {
      cout << "Bounded set pretty print" << endl;
      typename set_t::iterator it;
      for (it = this->begin();
	   it != this->end();
	   it++) {

	(*it)->print_me();
      }
	
      cout << "Pretty print no more" << endl;
    }
    
  protected:
    unsigned int s_size;
    
  }; // bounded_set
}; // namespace

#endif
