#include "test_generator.h"

using user::test_generator;


// The constructor
test_generator::test_generator(sc_module_name module_name ) : 
  sc_module (module_name),
  basic_module(),
  io1_in_port("io1_in"),
  io1_out_port("io1_out"),
  clk("clock_in")
{
  SC_THREAD( send_to_io );
  sensitive << clk.pos();

  io1_in_port(*this);
}

/**
 * Implementation of the receive_transaction method from the test_if
 * (since we are using sc_export for the ports connecting to the io
 * modules, we need to implement the methods localy.
 */
status_t test_generator::receive_transaction(const token_t* t) {
  dprint(9, "Test generator: receiving transaction\n");
  return basic_protocol::SUCCESS;
}



/**
 * Generates and sends test vectors to the io modules. Waits for
 * confirmation of successful receipt of the tokens before sending the
 * next one.
 */
void test_generator::send_to_io() {
   
  dprint(9, "Test generator is running...\n");
  
  int i=0;
  
  while (true) {
    
    if ( rand() & 1 ) {
      
      dprint(9, "%s: i = %d\n", name(), i);
      
      airline::request* r = new airline::request(i, 2, 3, 10, 4, 5, i);
      
      dprint(7, "%s: Sending a request with id %d with source %d\n", 
	     name(), r->get_request_id(), i);
      
      token_t* req = new token_t(r, basic_protocol::NEW);
      
      basic_protocol::status_t status = io1_out_port->receive_transaction(req);
      
      if (status == basic_protocol::SUCCESS) {
	dprint(8, "%s: request %d successfully sent to IO module\n", 
	       name(), r->get_request_id());
	i++;
      }
      else {
	dprint(8, "%s: request %d could not be sent to IO module at this time\n", 
	       name(), r->get_request_id());
	// We don't increment the counter because we will retry in a moment
      }
      
      dprint(9, "%s: Taking a break until the next clock tick.\n", name());
      wait(); // change to clock sensitivity...
    }
  }
}


